from django.urls import path
from django.conf import settings
from django.conf.urls.static import static

from . import views

app_name = 'device_status'
urlpatterns = [
    # path('', views.index, name='index'),
    path('', views.device_list, name='index'),
    path('all', views.device_list_all, name='index'),
    path('device_status/', views.device_list, name='status'),
    path('device_status/report', views.report, name='report'),
    path('devices', views.device_list, name='devices'),
    path('actions/<str:mac>', views.actions_json, name='actions'),
    path('actions/<str:mac>/pending', views.action_list, name='actions_pending'),
    path('actions', views.action_list, name='action_list'),
    path('action_report', views.action_report, name='action_report'),
    path('power_cycle/<str:device>', views.power_cycle, name='power_cycle_device'),
    # path('setup', views.client_setup),
] + static(settings.STATIC_URL, document_root=settings.STATIC_ROOT)



import subprocess
from uuid import getnode
import pkg_resources
import requests

# json as callable was introduced in version 1.0.0
pkg_resources.require("requests>=1.0.0")


mac = getnode()
mac = ':'.join(('%012x' % mac)[i:i+2] for i in range(0, 12, 2))
base_url = 'http://pimanager'
actions = requests.get(base_url + '/actions/'+mac)
for action in actions.json():
    print(action)
    arguments = action['command'].split(' ')
    output = ''
    try:
        output = subprocess.check_output(arguments, universal_newlines=True,
                                         stderr=subprocess.STDOUT)
        return_code = 0
    except subprocess.CalledProcessError as exception:
        return_code = exception.returncode
    except OSError as exception:
        output = exception.strerror
        return_code = 999

    print(output)

    requests.post(base_url + '/action_report', data={'action': action['id'],
                                                     'output': output,
                                                     'return_code': return_code})



from django.test import TestCase
from pimanager.models import Device, Action


class DeviceTestCase(TestCase):
    def setUp(self):
        Device.objects.create(mac='b8:27:eb:eb:a9:dd', name='utstilling-pi-01',
                              ip='10.0.4.221',
                              monitored_process='omxplayer.bin',
                              serial='8aeba9dd',
                              number=1,
                              last_seen='2019-01-01')

    def test_last_seen_class(self):
        device = Device.objects.get(serial='8aeba9dd')
        self.assertEqual(device.last_seen_class(), 'error')



from django.db import models
from datetime import datetime
from django.apps import apps


class Device(models.Model):
    mac = models.CharField(max_length=45, unique=True)
    name = models.CharField('Navn', max_length=200, blank=True, null=True)
    description = models.CharField('Beskrivelse', max_length=200, blank=True, null=True)
    message = models.CharField(max_length=200, blank=True, null=True)
    ip = models.GenericIPAddressField(blank=True, null=True)
    last_seen = models.DateTimeField('Sist sett', auto_now=False, blank=True, null=True)
    number = models.IntegerField('Nummer', blank=True, null=True)
    monitored_process = models.CharField(max_length=200, blank=True, null=True)
    uptime = models.CharField('Oppetid', max_length=200, blank=True, null=True)
    screen_status = models.CharField('Skjermstatus', max_length=200, blank=True, null=True)
    serial = models.CharField(max_length=50, blank=True, null=True,
                              unique=True,
                              verbose_name='Serienummer')
    model = models.CharField('Modell', max_length=200, blank=True, null=True)
    hide = models.BooleanField('Skjul', default=False)

    def __str__(self):
        return str(self.ip) + ' ' + str(self.name) + ' ' + str(self.description)

    class Meta:
        ordering = ['number']

    def interface(self):
        if apps.is_installed('switchinfo'):
            from switchinfo.models import Mac
            mac_address = self.mac.replace(':', '')
            mac = Mac.objects.get(mac=mac_address)
            if Mac.objects.filter(interface=mac.interface).count() > 1:
                return None
            else:
                return mac.interface

    def port(self):
        if apps.is_installed('switchinfo'):
            from switchinfo.models import Mac
            mac_address = self.mac.replace(':', '')
            mac = Mac.objects.get(mac=mac_address)
            return '%s %s' % (mac.interface.switch.name, mac.interface)

    def last_seen_class(self):
        if self.last_seen is None:
            return 'good'
        diff = datetime.today()-self.last_seen

        if diff.days > 0:
            return 'error'
        elif diff.seconds < 60*15:
            return 'good'
        elif diff.seconds < 3600*24:
            return 'warning'

        return diff.seconds

    def pending_actions(self):
        actions = Action.objects.filter(device=self, executed=None)
        return actions


class Action(models.Model):
    device = models.ForeignKey(Device, on_delete=models.CASCADE)
    command = models.CharField(max_length=200)
    added = models.DateTimeField(auto_now_add=True)
    executed = models.DateTimeField(auto_now=False, blank=True, null=True)
    return_code = models.IntegerField(blank=True, null=True)
    output = models.TextField(blank=True, null=True)



import datetime
import os
import re
from pprint import pprint

from django.apps import apps
from django.http import HttpResponse, JsonResponse
from django.shortcuts import get_object_or_404, redirect, render as django_render
from django.views.decorators.csrf import csrf_exempt

from pimanager.models import Action, Device


def render(request, template_name, context=None, content_type=None, status=None, using=None):
    version_file = os.path.join(apps.get_app_config('pimanager').path, 'VERSION')
    if os.path.exists(version_file):
        with open(version_file) as fp:
            version = fp.read().strip()
        version = version
        context['version'] = version
    else:
        context['version'] = 'development version'

    return django_render(request, template_name, context, content_type, status, using)


@csrf_exempt
def report(request):
    if request.method == "POST":
        number = re.match(r'.+pi\-([0-9]+)', request.POST['hostname'])

        if 'serial' not in request.POST:
            return HttpResponse('Missing serial number')
        dev, new = Device.objects.get_or_create(serial=request.POST['serial'],
                                                defaults={'name': request.POST['hostname'],
                                                          'ip': request.META['REMOTE_ADDR']})
        if number:
            dev.number = int(number.group(1))
        pprint(dev)
        dev.name = request.POST['hostname']
        if 'serial' in request.POST:
            dev.serial = request.POST['serial']
        if 'uptime' in request.POST:
            dev.uptime = request.POST['uptime']
        if 'tvservice' in request.POST:
            dev.screen_status = request.POST['tvservice']
        if 'model' in request.POST:
            dev.model = request.POST['model']
        if 'mac' in request.POST:
            dev.mac = request.POST['mac']

        dev.ip = request.META['REMOTE_ADDR']
        dev.last_seen = datetime.datetime.now()

        if dev.monitored_process:
            if 'processes' in request.POST:
                process = re.search(r'.+%s (.+)' % dev.monitored_process, request.POST['processes'])
                if process:
                    if dev.monitored_process == 'omxplayer.bin':
                        dev.message = 'Playing %s' % process.group(1)
                    elif dev.monitored_process == 'chromium-browser':
                        dev.message = '%s running' % dev.monitored_process
                    elif dev.monitored_process == 'fbi':
                        dev.message = '%s running' % dev.monitored_process
                    else:
                        dev.message = '%s running with argument %s' % (dev.monitored_process, process.group(1))
                else:
                    dev.message = '%s not found' % dev.monitored_process

        dev.save()
        if not new:
            return HttpResponse('Updated status for device %s %s' % (dev.number, datetime.datetime.now()))
        else:
            return HttpResponse('Added new device %s %s' % (dev.number, datetime.datetime.now()))

    else:
        return HttpResponse('Not a POST request')


def device_list(request, show_all=False):
    if not show_all:
        devices = Device.objects.filter(hide=False).order_by('number')
    else:
        devices = Device.objects.order_by('number')
    context = {
        'devices': devices,
    }
    return render(request, 'pimanager/devices.html', context)


def device_list_all(request):
    return device_list(request, True)


def actions_json(request, mac):
    device = get_object_or_404(Device, mac=mac)
    actions = Action.objects.filter(device=device, executed=None)

    return JsonResponse(list(actions.values()), safe=False)


def action_list(request, mac=None, serial=None, pending=True):
    if mac:
        actions = Action.objects.filter(device__mac=mac)
    elif serial:
        actions = Action.objects.filter(device__serial=serial)
    else:
        actions = Action.objects.all()
    if pending:
        actions.filter(executed=None)

    context = {'actions': actions}
    return render(request, 'pimanager/actions.html', context)


@csrf_exempt
def action_report(request):
    if request.method == "POST":
        action = get_object_or_404(Action, id=request.POST['action'])
        action.output = request.POST['output']
        action.return_code = request.POST['return_code']
        action.executed = datetime.datetime.now()
        action.save()
    return HttpResponse('ok')


def tv_off(request, device):
    command = 'tvservice -o'
    action = Action(device=device, command=command)
    action.save()
    return redirect('device_status:action_list')


def reboot(request, device):
    command = 'reboot'
    action = Action(device=device, command=command)
    action.save()


def client_setup(request):
    from django.templatetags.static import static
    url = static('pimanager/client_setup.sh')
    return redirect(url)


def power_cycle(request, device):
    # TODO: Integrate with switch backup
    return HttpResponse('Power cycle not implemented')



from django.contrib import admin

# Register your models here.

from .models import Action, Device


class DeviceAdmin(admin.ModelAdmin):
    list_display = ('name', 'description', 'mac', 'ip', 'last_seen', 'serial')


admin.site.register(Device, DeviceAdmin)


class ActionAdmin(admin.ModelAdmin):
    list_display = ('device', 'command', 'added', 'executed', 'return_code')
    list_filter = ('device', 'return_code')


admin.site.register(Action, ActionAdmin)



from django.core.management.base import BaseCommand

from pimanager.models import Action, Device


class Command(BaseCommand):
    help = 'Add action to device(s)'

    def add_arguments(self, parser):
        parser.add_argument('device', nargs='+', type=str)

    def handle(self, *args, **options):
        if not options['device'][0] == 'all':
            devices = Device.objects.filter(name=options['device'][0])
        else:
            devices = Device.objects.all()
        for device in devices:
            command = 'wget -O /tmp/client_setup.sh http://pimanager/static/pimanager/client_scripts/client_setup.sh'
            action = Action(device=device, command=command)
            action.save()
            command = 'sh /tmp/client_setup.sh'
            action2 = Action(device=device, command=command)
            action2.save()



import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

if not os.path.exists('VERSION'):
    version = '0'
else:
    with open('VERSION', 'r') as fp:
        version = fp.read().strip()

setup(
    name='pimanager',
    version=version,
    packages=find_packages(),
    include_package_data=True,
    license='GPL',
    description='A django app to monitor status and manage Raspberry Pi devices',
    long_description=README,
    url='https://github.com/datagutten/pimanager',
    author='Anders Birkenes',
    author_email='datagutten@datagutten.net',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.1',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ], install_requires=['django']
)



# Generated by Django 2.1.2 on 2019-07-31 10:43

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Action',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('command', models.CharField(max_length=200)),
                ('added', models.DateTimeField(auto_now_add=True)),
                ('executed', models.DateTimeField(blank=True, null=True)),
                ('return_code', models.IntegerField(blank=True, null=True)),
                ('output', models.TextField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Device',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('mac', models.CharField(max_length=45, unique=True)),
                ('name', models.CharField(blank=True, max_length=200, null=True, verbose_name='Navn')),
                ('description', models.CharField(blank=True, max_length=200, null=True, verbose_name='Beskrivelse')),
                ('message', models.CharField(blank=True, max_length=200, null=True)),
                ('ip', models.GenericIPAddressField(blank=True, null=True)),
                ('last_seen', models.DateTimeField(blank=True, null=True, verbose_name='Sist sett')),
                ('number', models.IntegerField(blank=True, null=True, verbose_name='Nummer')),
                ('monitored_process', models.CharField(blank=True, max_length=200, null=True)),
                ('uptime', models.CharField(blank=True, max_length=200, null=True, verbose_name='Oppetid')),
                ('screen_status', models.CharField(blank=True, max_length=200, null=True, verbose_name='Skjermstatus')),
                ('serial', models.CharField(blank=True, max_length=50, null=True, unique=True, verbose_name='Serienummer')),
                ('model', models.CharField(blank=True, max_length=200, null=True, verbose_name='Modell')),
                ('hide', models.BooleanField(default=False, verbose_name='Skjul')),
            ],
            options={
                'ordering': ['number'],
            },
        ),
        migrations.AddField(
            model_name='action',
            name='device',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='pimanager.Device'),
        ),
    ]



from django.core.management.base import BaseCommand

from pimanager.models import Action, Device


class Command(BaseCommand):
    help = 'Update player on device(s)'

    def add_arguments(self, parser):
        parser.add_argument('device', nargs='+', type=str)

    def handle(self, *args, **options):
        if not options['device'][0] == 'all':
            devices = Device.objects.filter(name=options['device'][0])
        else:
            devices = Device.objects.all()
        for device in devices:
            command = 'wget -O /home/pi/omxloop.py http://pimanager/static/pimanager/client_files/omxloop/omxloop.py'
            action = Action(device=device, command=command)
            action.save()
            command = 'wget -O /media/no_media.jpg http://pimanager/static/pimanager/client_files/omxloop/no_media.jpg'
            action2 = Action(device=device, command=command)
            action2.save()



import re
import subprocess
from uuid import getnode
import requests

import socket
hostname = socket.gethostname()


# https://stackoverflow.com/questions/159137/getting-mac-address
mac = getnode()
mac = ':'.join(('%012x' % mac)[i:i+2] for i in range(0, 12, 2))
# print (mac)

ifconfig = subprocess.check_output(['/sbin/ip', 'addr'], universal_newlines=True)

#m = re.search('[a-f0-9]{2}:[a-f0-9]{2}:[a-f0-9]{2}:[a-f0-9]{2}:[a-f0-9]{2}:[a-f0-9]{2}', ifconfig)
#mac = m.group()

ps_aux = subprocess.check_output(['/bin/ps', 'aux'])
tvservice = subprocess.check_output(['tvservice', '-s'])
try:
    uptime = subprocess.check_output(['uptime', '-p'])
    upsince = subprocess.check_output(['uptime', '-s'])
except subprocess.CalledProcessError:
    pattern = r'.+(up.+),\s+[0-9]+ user.+'
    uptime = subprocess.check_output(['uptime'])
    uptime = re.sub(pattern, r'\1', uptime.decode('utf-8'))
    print(uptime)
    upsince = ''

file = open('/proc/cpuinfo', 'r')
cpuinfo = file.read()
groups = re.search(r'Serial\s+: .*?([1-9a-f][0-9a-f]+)', cpuinfo)
if groups:
    serial = groups.group(1)
else:
    serial = None

try:
    file_model = open('cat /sys/firmware/devicetree/base/model', 'r')
    model = file.read()
except IOError:
    model = ''


action = requests.post('http://pimanager/device_status/report',
                       data={'mac': mac,
                             'processes': ps_aux,
                             'ifconfig': ifconfig,
                             'hostname': hostname,
                             'tvservice': tvservice,
                             'uptime': uptime,
                             'upsince': upsince,
                             'serial': serial,
                             'model': model}
                       )
print(action.text)



#!/usr/bin/python
import subprocess
import os
import glob

path = '/media/usb/'
pidfile = open('/var/run/omxloop.pid', 'w')

# Turn off display
# subprocess.call(['tvservice', '-o'])

on = True
file_list = glob.glob(os.path.join(path, '*.mp4'))
file_list.sort()

if len(file_list) == 0:
    # Turn on display
    # subprocess.call(['tvservice', '-p'])
    cmd = ["fbi", '-a', '-fitwidth', '-t', '7', '-noverbose', '-blend', '400', '-T', '1']

    file_list_0 = glob.glob(os.path.join(path, '*.jpg'))
    file_list_1 = glob.glob(os.path.join(path, '*.png'))
    file_list_2 = glob.glob(os.path.join(path, '*.tif'))
    fl = file_list_0 + file_list_1 + file_list_2
    if len(fl) == 0:
        # https://edv-huber.com/index.php/problemloesungen/15-custom-splash-screen-for-raspberry-pi-raspbian
        subprocess.run(['fbi', '-a', '-T', '1', '-noverbose', '/media/no_media.jpg'])
    else:
        fl.sort()
        try:
            a = subprocess.call(cmd + fl)
        except KeyboardInterrupt:
            pass
else:

    sub_list = [None] * len(file_list)
    for idx, infile in enumerate(file_list):
        bname = os.path.splitext(infile)[0]
        if os.path.exists(bname + '.srt'):
            sub_list[idx] = bname + '.srt'

    # Turn on display
    subprocess.call(['tvservice', '-p'])
    while on:
        for idx, infile in enumerate(file_list):
            try:
                if sub_list[idx] is not None:
                    args = ["omxplayer", '--subtitles', sub_list[idx], infile]
                else:
                    args = ["omxplayer", infile]
                a = subprocess.call(args)
            except KeyboardInterrupt:
                on = False
                break
